import json
import re
import secrets
import urllib.parse

import pytest
import responses


@pytest.fixture
def mock_get_signin_token(requests_mock):
    def request_callback(request):
        headers = {"Content-Type": "application/json"}
        token = secrets.token_urlsafe()
        return (200, headers, json.dumps({"SigninToken": token}))

    requests_mock.add_callback(
        responses.GET,
        re.compile("https://[a-z0-9-]+\\.signin\\.aws\\.amazon\\.com/federation"),
        callback=request_callback,
        content_type="application/json",
        match=[
            responses.matchers.query_param_matcher(
                {"Action": "getSigninToken"}, strict_match=False
            )
        ],
    )


def test_configuration_file_required(cli_invoke):
    "Command fails if no configuration file is present."
    result = cli_invoke("console")
    assert result.exit_code != 0
    assert "Failed to locate a configuration file" in result.stderr


def test_basic_usage(
    cli_invoke, fake_cli_config, mock_google_user_info_endpoint, mock_get_signin_token
):
    result = cli_invoke("console", "--no-browser")
    assert result.exit_code == 0
    parts = urllib.parse.urlsplit(result.stdout.strip())
    assert parts.scheme == "https"
    assert parts.netloc.endswith(".signin.aws.amazon.com")
    assert parts.path == "/federation"
    query = urllib.parse.parse_qs(parts.query)
    assert query["Action"] == ["login"]
